#include <stdio.h>
#include <unistd.h>

// A global variable.
int counter = 0;

// The parent process begins executing at main().  There is only
// a single thread of execution in this process.
int main(int argc, char **argv) {
  pid_t parent_pid = getpid();
  pid_t child_pid;

  printf("(parent):  my pid is %d\n", (int) parent_pid);
  printf("(parent):  my counter has value %d\n", counter);
  printf("(parent):  my counter lives at (virtual) address %p\n\n", &counter);

  // Invoke the fork() system call to create a child process.  Note
  // that fork(), if successful, returns twice: once into the parent,
  // and once into the child.  The return value of fork is -1 if there
  // was an error, 0 if this is the child, and the PID of the child
  // (i.e., something other than -1 or 0) if this is the parent.
  child_pid = fork();
  if (child_pid == -1) {
    printf("The fork() system call failed!\n");
    return -1;
  }

  // fork() succeeded.  Both the parent and child process now exist,
  // and could be scheduled anytime.  Both the parent and child will
  // begin executing here.
  counter++;

  // Do something different in the child and the parent.
  if (child_pid == 0) {
    // child_pid == 0  ==>  this is the child process
    child_pid = getpid();   // OK, I'm the child, so grab my actual pid.
    printf("(child):  my pid is %d, my parent's pid is %d\n",
           child_pid, parent_pid);
    printf("(child):  my counter has value %d\n", counter);
    printf("(child):  my counter lives at (virtual) address %p\n\n",
           &counter);
    return 0;
  } else {
    // child_pid != 0 ==>  this is the parent process
    printf("(parent):  my child's pid is %d\n", child_pid);
    printf("(parent):  my counter has value %d\n", counter);
    printf("(parent):  my counter lives at (virtual) address %p\n\n",
           &counter);
  }

  // Done with the if/else.  Both the child and parent processes continue
  // executing here.
  sleep(5);  // go to sleep for 5 seconds
  return 0;  // Return 0 to indicate successful execution.  The OS
             // will destroy the process once it returns out of main().
}
